/* interfaces.c */
/* ITOGAWA's Slitherlink Solver */

/*
 Ϥ˴طؿ
*/

#include "slink.h"

static char griph[5][8] = 
{
    { 0x81, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81 }, /* . */
    { 0x81, 0x18, 0x24, 0x24, 0x24, 0x24, 0x18, 0x81 }, /* 0 */
    { 0x81, 0x08, 0x18, 0x08, 0x08, 0x08, 0x08, 0x81 }, /* 1 */
    { 0x81, 0x18, 0x24, 0x0c, 0x10, 0x20, 0x3c, 0x81 }, /* 2 */
    { 0x81, 0x3c, 0x04, 0x18, 0x04, 0x24, 0x18, 0x81 }  /* 3 */
};

void parse_commandline   (int argc, char *argv[])
{
    number ind = 1;

    while(ind < argc)
    {
	if (argv[ind][0] == '-')
	{
	    switch (argv[ind][1])
	    {
	    case 'a':
		opt_solveall = TRUE;   
		break;
	    case 'b':
		print_solution_hook = print_solution_block;
		break;
	    case 'd':
		ind++;
		if (ind >= argc)
		{
		    fprintf(stderr, "usage: %s [-d debuglevel]", argv[0]);
		    fprintf(stderr, "[-a -b -g -l -w][filename]\n");
		    exit(1);
		}
		debug = atoi(argv[ind]);
		break;
	    case 'g':
		opt_grphout = TRUE;   
		break;
	    case 'l':
		print_solution_hook = print_solution_line;
		break;
	    case 'w':
		opt_w = TRUE;   
		break;
	    default:
		fprintf(stderr, "usage: %s [-d debuglevel]", argv[0]);
		fprintf(stderr, "[-a -b -g -l -w][filename]\n");
		exit(1);
		break;
	    }
	}
	else
	{
	    problem = argv[ind];
	    if ((input = fopen(argv[ind], "r")) == NULL)
	    {
		fprintf(stderr, "slink: couldn't open %s.\n", problem);
		exit(1);
	    }
	}
	ind++;
    }
}

void read_problem(FILE *input)
{
    int letter;

    while((letter = getc(input)) != EOF)
    {
        switch (letter)
	{
	case '#':
	    while(getc(input) != '\n')
		;
	    break;
  	case '0':
	case '1':
	case '2':
	case '3':
	case '.':
  	    sides[y+1][x+1] = (letter == '.' ? BLANK : letter - '0');
	    if (x > XMAX)
	    {
		fprintf(stderr, "Error: %s is too large.\n", problem);
		exit(1);
	    }
	    x++;
	    cmax++;
	    break;
	case '\n':
	    if (y > YMAX)
	    {
		fprintf(stderr, "Error: %s is too large.\n", problem);
      		exit(1);
	    }
	    if (x > 1) y++; 
            x = 1;
	    break;
	default:
	    break;
	}
    }
    y--; x = cmax/y; cmax++;
    fclose(input);
}

void print_problem(void)
{
    geometry i, j;

    for (i=2; i <= y+1; i++)
    {
	for(j=2; j <= x+1; j++)
	    switch (sides HERE)
            {
	    case 0:
	    case 1:
	    case 2:
	    case 3:
		fprintf(stderr, " %c", sides HERE +'0');
		break;
	    case BLANK:
		fprintf(stderr, " .");
		break;
	    default:
		fprintf(stderr, " ?");
		break;
	    }
	fprintf(stderr, "\n");
    }
    fprintf(stderr, "\n");
}

/* ǥХåѤδؿ */
void print_activity(void)
{
    geometry i, j;

    for (i=1; i <= y+2; i++)
    {
	for(j=1; j <= x+2; j++)
	    if (the->activity HERE)
		fprintf(stderr, " *");
	    else
		fprintf(stderr, " /");
	fprintf(stderr, "\n");
    }
    fprintf(stderr, "\n");
}

void print_solution(void)
{
    FILE *output = the->degfree ? stderr : stdout;

    fprintf(stderr, "\n");
    if (opt_grphout && !the->degfree)
	print_solution_grph();
    else
	(*print_solution_hook)(output);

    if (!the->degfree)
    {
	solutions++;
	fprintf(stderr, "I solved the problem in level %d.\n", level);
        longjmp(the->jb, SUCCESS);
    }
}

void print_solution_block(FILE *output)
{
    geometry i, j;

    for (i=2; i <= y+1; i++)
    {
	for(j=2; j <= x+1; j++)
	    switch (the->color HERE)
            {
	    case BLACK:
		fprintf(output, "%2s", "*");
		break;
	    case WHITE:
		fprintf(output, "%2s", "/");
		break;
	    default:
		fprintf(output, "%2s", ".");
		break;
	    }
	fprintf(output, "\n");
    }
    fprintf(output, "\n");
}

void print_solution_line(FILE *output)
{
    number i, j;
    colour p, q;

    for (i=0; i <= y*2; i++)
    {
	for(j=0; j <= x*2; j++)
	    if (i & 1)
	    {
		if (j & 1)
		{
		    fprintf(output, " %c", sides[(i+3)>>1][(j+3)>>1] == -1 ?
			    '.' : sides[(i+3)>>1][(j+3)>>1] + '0');
		}
		else
		{
		    p = the->color[(i+3)>>1][(j+2)>>1];
		    q = the->color[(i+3)>>1][(j+4)>>1];
		         if (p ==  q) fprintf(output, " x");  
		    else if (p == -q) fprintf(output, " |");  
		    else              fprintf(output, "  ");
		}
	    }
	    else
	    {
		if (j & 1)
		{
		    p = the->color[(i+2)>>1][(j+3)>>1];
		    q = the->color[(i+4)>>1][(j+3)>>1];
		         if (p ==  q) fprintf(output, " x");  
		    else if (p == -q) fprintf(output, " -");  
		    else              fprintf(output, "  ");
		}
		else
		{
		    fprintf(output, " +");
		}
	    }
	fprintf(output, "\n");
    }
    fprintf(output, "\n");
}

void print_solution_grph(void)
{
    number i, j, k;
    char buf[8][XMAX+2];

    buf[0][0  ] = 0x01;
    buf[0][x+1] = 0x80;
    for(k=1; k <= 6; k++)
    {
	buf[k][0  ] = 0x00;
	buf[k][x+1] = 0x00;
    }
    buf[7][0  ] = 0x01;
    buf[7][x+1] = 0x80;

    printf("P4\n#Answer to %s\n#Created by slink\n", problem);
    printf("%d %d\n", (x+2)*8, (y+2)*8);
    
    for(j=7*(x+2); j; j--)
	putchar (0x00);
    putchar(0x01);
    for(j=x; j; j--)
	putchar (0x81);
    putchar(0x80);

    for (i=1; i <= y; i++)
    {
	for(j=1; j <= x; j++)
	    for(k=0; k <= 7; k++)
	    {
		buf[k][j] = griph[(sides [i+1][j+1])+1][k];
		if (the->color [i+1][j+1] == BLACK) buf[k][j] ^= 0xff;
	    }
	for(k=0; k <= 7; k++)
	    for(j=0; j <= x+1; j++)
		putchar(buf[k][j]);
    }

    putchar(0x01);
    for(j=x; j; j--)
	putchar (0x81);
    putchar(0x80);
    for(j=7*(x+2); j; j--)
	putchar (0x00);

}

